// ModelSwarm.h

#import "Consumer.h"
#import "Vendor.h"
#import "MeanPrice.h"
#import "ConsumerDescription.h"

#import <swarmobject/Swarm.h>
#import <activity.h>           // activity.h is required in ModelSwarm,
#import <simtools.h>           // that in turn is an object, to have access
                               // to the object modelSchedule

#import <collections.h>        // required to manage the lists

#import "Shuffler.h" // to give a random order to the consumers; this is a 
                     // class that I derive from sss,
                     // see www.santafe.edu/projects/swarm

#import "Keeper.h"   // to pick up the first element from a list
                     // and rotate all the list

@interface ModelSwarm: Swarm {  // notice: it is a Swarm, not a SwarmObject

  id aConsumer, aVendor;     // objects derived from Consumer and Vendor
                             // using the buildObjects method
  
  id consumerList, consumerList0;
                               // Consumer lists, one of them is frozen                              
  
  id vendorList;              // Vendor list
  
  Shuffler  * shuffler;         // a shuffler from Shuffler
  Keeper    * keeper;           // a  keeper   from Keeper
  MeanPrice * meanPrice;        // a mean-computer meanPrice from
                                // MeanPrice
  ConsumerDescription * consumerDescription; 
                                // array of the data needed to plot
                                // the price situation for
                                // each consumer

  id modelActions1;            // object containing the messages to be sent
                               // to different objects at each loop 

  id modelSchedule;            // object containing the actions to be done

  float theLevel;              // values of ModelSwarm (static) quantities
  float minPrice, maxPrice, minStartPrice, maxStartPrice;
  int agentNumber;

  int use_printf;              // when =1  printf will be used on the open terminal
  
}

// + means that this is a method of the parent object
+createBegin: (id) aZone;      // creation of the object ModelSwarm
-createEnd;                    // in two steps, in the memory location pointed
                               // by aZone; for the previous objects
                               // (Consumer and Vendor) I have not 
                               // directly managed the creation methods
                               // that are inherited from the parent object

-buildObjects;                  // creation of the objects
-buildActions;                  // indication of the actions

-activateIn: (id) swarmContext; // pasting of actions and objects
                                // on the pointer swarmContext location
                                // in observerSwarm, in a location that
                                // will not be simply nil
                                
                                // the activation is done
                                // by the observerSwarm in self 

-getMeanPrice;                  // it passes to the Observer the location
                                // of the object meanPrice 

-getConsumerList0;              // idem for the object consumerList0

-(int) getAgentNumber;          // it passes to Observer the number
                                // of the agents
                                
-getConsumerDescription;        // it passes to the Observer the location 
                                // of the object consumerDescription


@end
