// ModelSwarm.m

#import "ModelSwarm.h"

@implementation ModelSwarm

// remark from simpleObserverBug/ModelSwarm.m
// These methods provide access to the objects inside the ModelSwarm.
// These objects are the ones visible to other classes via message call.
// In theory we could just let other objects use Probes to read our state,
// but message access is frequently more convenient.

-getMeanPrice {
  return meanPrice;
}

-getConsumerList0 {
  return consumerList0;
}

-(int) getAgentNumber {
  return agentNumber;
}

-getConsumerDescription {
  return consumerDescription;
}

+createBegin: (id) aZone { // aZone corresponds to the result of [self getZone]
                           // in the observer

  ModelSwarm * obj; // obj is a pointer to an object of the ModelSwarm kind
                    // and it is defined only within this method;
                    // as a matter of fact it does not exist in the interface
                    // of ModelSwarm.h

  obj = [super createBegin: aZone]; // it calls the super-class (here Swarm)
                                    // from which it gets the location of the object
                                    // obj, i.e. our ModelSwarm

  obj->theLevel=25.0;  // (default) values of the parameters of the simulation
  obj->minPrice=0.0;   // having put them here, implicitely means that
  obj->maxPrice=100.0; // they will not be modified during the simulation

  obj->agentNumber=10; // number of the agents, both Consumers and Vendors
  obj->minStartPrice=70.0;
  obj->maxStartPrice=90.0;

  obj->use_printf=1;   // it uses printf in the various objects; it is modified, in case,
                       // in "consumer.setup"

  // object->var it refers to the example of var belonging to that 
  // example of object, without requiring a method;

  return obj;          // it returns the location of the object that has been created
                       // it might also be only a temporary location
                       // since the final one comes form the createEnd
}

-createEnd {
  return [super createEnd];
}

-buildObjects {

  int i;

  // I look for possible variations of the default values in consumer.setup
  // notice I have not created ObjectLoader, that is here a Swarm object

  [ObjectLoader load: self fromFileNamed: "consumer.setup"];
// theLevel -  it raises the number of purchases when it is increased; when it
// is greater or equal to 100, we have as many purchases as loops

  // at first I create an object to give a random order to the 
  // agent list
  shuffler = [Shuffler createBegin: [self getZone]];
  [shuffler setUniformRandom: uniformIntRand];  // uniformIntRand is
                                                // an example of o. self-created
                                                // in Swarm
  shuffler = [shuffler createEnd];

  // and an object to pick up the location of the first object from a list 
  // and rotate the list

  keeper   = [Keeper   createBegin: [self getZone]];
  keeper   = [keeper   createEnd];

  // creation of an object to compute the means at each loop

  meanPrice = [MeanPrice createBegin: [self getZone]];
  [meanPrice usePrintf: use_printf]; // to print or not
  meanPrice = [meanPrice createEnd];

  // creation and setting-up of the array containing the graphical information
  // to plot the "price side" of the consumers

  consumerDescription = [ConsumerDescription createBegin: [self getZone]];
  [consumerDescription setSizeX: agentNumber*30 Y: 200];
  [consumerDescription setMinPrice: minPrice MaxPrice: maxPrice];
  consumerDescription = [consumerDescription createEnd];

  // creation of the objects-agents

  vendorList =[List create: [self getZone]]; // it creates a Vendor list
                                              // in a self-defined Zone

for (i=1;i<=agentNumber;i++){  // it creates the Vendors (10 or a different
                               // number whenever specified in consumer.setup)


  aVendor = [Vendor createBegin: [self getZone]];       // an example 
                                                           // of seller
                                                           // with double
                                                           // init. (note 1)
  [aVendor setMinPrice: minPrice MaxPrice: maxPrice      // absolute
minStartPrice: minStartPrice maxStartPrice: maxStartPrice];// and starting range
                                                           // for the price,
                                                           // see Vendor.h
  [aVendor setMemory];
  [aVendor setNumber: i andPrintf: use_printf]; 
                               // here becouse it will not vary
  aVendor = [aVendor createEnd];                                 

  [vendorList  addLast: aVendor];

}

  consumerList =[List create: [self getZone]]; // it creates the consumer list
                                               // in a self-defined Zone
  consumerList0=[List create: [self getZone]]; // it creates the frozen consumer
                                               // list; on the contrary, the first
                                               // one will receive a random order


for (i=1;i<=agentNumber;i++){  // it creates the Consumers (10 or a different
                               // number whenever specified in consumer.setup) 
                               // and put them in consumerList

  aConsumer = [Consumer createBegin: [self getZone]];   // an example
                                                        // of consumer
  [aConsumer setNumber: i andPrintf: use_printf];
  [aConsumer setMemory];
  [aConsumer setMinPrice: minPrice MaxPrice: maxPrice];
                              // here becouse they will not vary
  aConsumer = [aConsumer createEnd];

  [aConsumer setTime: 0L numberOfTransactionCounter: 0L]; // 0L = long
  [aConsumer setBuyLevel: theLevel]; // price threshold below which it buys
                                     // (the condition is <= ) 

  [aConsumer setVendorList: vendorList]; // it passes to aConsumer the location of
                                           // vendorList
  [aConsumer setKeeper: keeper];           // keeper
  [aConsumer setMeanPrice: meanPrice];     // meanPrice 
  [aConsumer setConsumerDescription:       // consumerDescription
                      consumerDescription];

  [consumerList  addLast: aConsumer];
  [consumerList0 addLast: aConsumer]; // frozen version of the list

}




  return self; // end of buildObjects
}

-buildActions {

  // we create a series of actions for aConsumer (via a list) and
  // aVendor

  // ActionGroup to contain messages for the Consumer list
  modelActions1 = [ActionGroup create: [self getZone]];

  [modelActions1 createActionTo: consumerDescription message: M(clear)];

  [modelActions1 createActionForEach: vendorList message: M(chooseRange)];

  [modelActions1 createActionTo: shuffler message: M(shuffleList:) :
                                 consumerList];        
  [modelActions1 createActionTo: shuffler message: M(shuffleList:) :
                                 vendorList];         // random reordering
                                                       // of the two lists
                                                       // notice M with an 
                                                       // arg. (it is a
                                                       // pointer)

  [modelActions1 createActionTo: meanPrice message: M(setZero)];

  [modelActions1 createActionForEach: consumerList message: M(negotiate)];

  [modelActions1 createActionTo: meanPrice message: M(findGlobalMean)];
  [modelActions1 createActionTo: meanPrice message: M(findTransactionMean)];

  [modelActions1 createActionForEach: consumerList0 message: 
                                M(reportStatistics)]; // it uses the frozen list
  [modelActions1 createActionForEach: consumerList  message: 
                                M(updateLevel)];

  modelSchedule = [Schedule createBegin: [self getZone]]; // it creates an
                                                          // example-object
                                                          // for the Schedule class

  [modelSchedule setRepeatInterval: 1]; // each action loop will have only
                                        // one time
  modelSchedule = [modelSchedule createEnd];

 [modelSchedule at: 0 createAction: modelActions1]; // notice, it uses
                                                    // modelActions1

// the end is not controlled; terminate via control panel
  return self;

}
  
-activateIn: (id) swarmContext {  // the swarmContext of the main is nil
                                  // here it is the result of [self getZone]
                                  // in modelSwarm

  [super activateIn: swarmContext]; // message to the class
  [modelSchedule activateIn: self]; // message that activates modelSchedule
  return [self getActivity];        // it returns the result of getActivity
                                    // applied to self

// mysterious, but repetitive, anyway

}

@end
 
// (1) notice the double create, which is not defined in Vendor.m and .h, 
//     since it is inherited from SwarmObject; createEnd is addressed to the example
//     (instance) aVendor, while createBegin is addressed to the class, in the
//     instruction that creates the example; the methods regarding variables that will
//     not be modified anymore should be placed between Begin and End
