// ObserverSwarm.m

#import "ObserverSwarm.h"

@implementation ObserverSwarm

+createBegin: (id) aZone {   // aZone in the main is globalZone that,
                             // in the current Swarm setup, is equal to nil

// the creation of the observerSwarm in the main (by create; here we have
// createBegin, but it is an alias) corresponds to the +createBegin
// procedure ( + leans it against the super-object, since
// the local object does not exist yet) that is developped here as it is
// in ModelSwarm.m (here and in the other applications) I think basically
// in order to have a direct way (via the -> to the object obj) to define
// the initial variables for the application, without creating any method,
// as I say in ModelSwarm.m
// to test: cancel all this block, redefine below the assigantion
// of the frequency displaySchedule and cancel in
// ModelSwarm.h the +createBegin; everything works, exactly as the creation
// of the other objects


  ObserverSwarm * obj;

  // createBegin: here we set up the simulation default parameters.

  // Superclass createBegin to allocate ourselves.

  obj = [super createBegin: aZone];

  // Fill in the relevant parameters (only one, in this case).

  obj->displayFrequency = 1;  // see below "remarks on the schedules"

  return obj;
}

-createEnd {
  return [super createEnd];
}

-buildObjects {
  id modelZone;                                   // zone for model.

  [super buildObjects];

  // First, we create the model that we're actually observing. The
  // model is a subswarm of the observer. We also create the model in
  // its own zone, so storage is segregated.

  modelZone = [Zone create: [self getZone]];
  modelSwarm = [ModelSwarm create: modelZone];

  // Instruct the control panel to wait for a button event.
  // We halt here until someone hits a control panel button.


  [controlPanel setStateStopped]; // controlPanel is an object that is not created
                                  // like for the objects generating random numbers
                                  // if we cancel this one, nothing happens;
                                  // the control panel is switched on
                                  // by the go message that the observerSwarm
                                  // receives in the main and that understands
                                  // becouse it inherits from GUISwarm

  // OK - the user said "go" so we're ready to start

  [modelSwarm buildObjects];


  // mean diagram, Notice: after the above build, otherwise it does not
  // find the selector of the object created by modelSwarm
  meanGraph = [EZGraph createBegin: [self getZone]];
  [meanGraph setTitle: "price means"];
  [meanGraph setAxisLabelsX: 
    "ages: at each age, all the consumers have an offer" Y: 
    "mean values"];
  meanGraph = [meanGraph createEnd];

  [meanGraph createSequence: "Global mean"
  withFeedFrom: [modelSwarm getMeanPrice]
  andSelector: M(getGlobalMean)] ;

  [meanGraph createSequence: "Transaction mean"
  withFeedFrom: [modelSwarm getMeanPrice]
  andSelector: M(getTransactionMean)] ;


  // diagram of the means, alternative way, computing directly the mean
  // from a list of prices (message getPrice to the list consumerList0)
  // and selecting the min. and max values that are contained each time
  // in the list
  mean2Graph = [EZGraph createBegin: [self getZone]];
  [mean2Graph setTitle: "Price mean, alternative way"];
  [mean2Graph setAxisLabelsX: 
    "Ages: at each age all the consumers have an offer" Y: 
    "Mean values"];
  mean2Graph = [mean2Graph createEnd];

  [mean2Graph createAverageSequence: "Global mean"
  withFeedFrom: [modelSwarm getConsumerList0]
  andSelector: M(getPrice)] ;

  [mean2Graph createMinSequence: "Global min price"
  withFeedFrom: [modelSwarm getConsumerList0]
  andSelector: M(getPrice)] ;

  [mean2Graph createMaxSequence: "Global max price"
  withFeedFrom: [modelSwarm getConsumerList0]
  andSelector: M(getPrice)] ;

  // display of the price behaviour of each consumer
  
  // colours, bound to the numbers from 0 to 255; for the colour names, use
  // the command showrgb in one's own shell; e.g. showrgb | less
  colorMap = [XColormap createBegin: [self getZone]];
  colorMap = [colorMap createEnd];
  [colorMap setColor: 0 ToName: "white"]; // notice, they produce an error 
                                          // if put above createEnd
  [colorMap setColor: 11 ToName: "LightYellow2"];
  [colorMap setColor: 12 ToName: "pink"];
  [colorMap setColor: 2  ToName: "blue"];
  [colorMap setColor: 3  ToName: "red"];

// below raster and widget are used in the meaning given by
// the Concise Oxford Dictionary
// raster  n. 
//  a pattern of scanning lines for a cathode-ray tube picture.
// widget  n. colloq. 
//  any gadget or device.

  // 2d window, what follows comes (with some variations) from simpleObserverBug 
  // of the tutorials
  // Next, create a 2d window for display, set its size, zoom factor, title.
  // ZoomRaster needs a squared picture; I use Raster
  consumerRaster = [Raster createBegin: [self getZone]];
  consumerRaster = [consumerRaster createEnd];
  // if the following instructions are put above createEnd they produce
  // a runtime error
  [consumerRaster setColormap: colorMap];
//  [consumerRaster setZoomFactor: 4]; // it works only with ZoomRaster
  [consumerRaster setWidth: [modelSwarm getAgentNumber]*30 // three 'columns' for
                                                           // each agent
                                                           // 10 dots for
                                                           // each column
                  Height: 200]; // fixed height
  [consumerRaster setWindowTitle: 
                  "Prices for each consumer"];
  [consumerRaster pack];      // draw the window (Tcl command).

  // Now create a Value2dDisplay: this is a special object that will
  // display arbitrary 2d value arrays on a given Raster widget.
  // or, in case, also create an Object2dDisplay: this object draws
  // objects on a Raster widget for us.

  consumerDisplay = [Value2dDisplay createBegin: [self getZone]];
  [consumerDisplay setDisplayWidget: consumerRaster Colormap: colorMap];
  [consumerDisplay setDiscrete2dToDisplay: [modelSwarm getConsumerDescription]];
  consumerDisplay = [consumerDisplay createEnd];

  return self;
}

-buildActions {

  [super buildActions];

  // First, let our model swarm build its own schedule.

  [modelSwarm buildActions];

  displayActions = [ActionGroup create: [self getZone]];

// from 1.0.4 actionCache it gets the messages of doTkEvents
// instead of controlPanel; actionCache is self-created and inherits
// from ActionCache
// replace the two following, for 1.0.3
//  [displayActions createActionTo: controlPanel message: M(doTkEvents)];
  [displayActions createActionTo: actionCache  message: M(doTkEvents)];
  [displayActions createActionTo: meanGraph    message: M(step)];
  [displayActions createActionTo: mean2Graph   message: M(step)];

  [displayActions createActionTo: consumerDisplay message: M(display)];
  // but it is also necessary to order to the Raster to redraw itself
  [displayActions createActionTo: consumerRaster message: M(drawSelf)];

  // And the display schedule. Note the repeat interval is set from our
  // own Swarm data structure. Display is frequently the slowest part of a
  // simulation, so redrawing less frequently can be a help.

  displaySchedule = [Schedule createBegin: [self getZone]];
  [displaySchedule setRepeatInterval: displayFrequency]; // note frequency!
//  [displaySchedule setRepeatInterval: 1]; // activate instead of the
                                            // preceding one if the
                                            // definition in createBegin
                                            // is cancelled; see above

  displaySchedule = [displaySchedule createEnd];
  [displaySchedule at: 0 createAction: displayActions];


// remarks on the schedules

// there is only one time tick; moreover, if there are empty ticks - i.e.
// without "[displaySchedule at:" neither in the observer nor in the model, they are skipped

// if the model has only one step the frequency of the observer means that 
// the observer makes n steps (with the n of the model), but only at the step 
// 0 of n, and then when, once at n, it starts working

// more complex example: if the frequency is 10 in the observer and the steps
// of the model are 2, we will have a 'd' display each 5 complete loops 
// (of the data, updated "at the a step" of the model)


// time  0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19
// model a  b  a  b  a  b  a  b  a  b  a  b  a  b  a  b  a  b  a  b
// obs   d  0  0  0  0  0  0  0  0  0  d  0  0  0  0  0  0  0  0  0

// in this experiment displayFrequency is 2 in order to have a step 
// every two where the Observer does not activate the EZgraphs' 
// (when the model has not updated the concerning data yet)

// a special remark: if in the model or in the observer any printfs' 
// not closed by \n are used, they may seem to be asynchronous; this C 
// does not produce out on stdout until an \n is given

  return self;
}

-activateIn: (id) swarmContext {

// activateIn: - activate the schedules so they're ready to run.
// The swarmContext argument has to do with what we were activated *in*.
// Typically the ObserverSwarm is the top-level Swarm, so it's activated
// in "nil". But other Swarms and Schedules and such will be activated
// inside of us.

  [super activateIn: swarmContext];

  // Activate the model swarm in ourselves. The model swarm is a
  // subswarm of the observer swarm.

  [modelSwarm activateIn: self];

  // Now activate our schedule in ourselves. This arranges for the
  // execution of the schedule we built.

  [displaySchedule activateIn: self];

  // Activate returns the swarm activity - the thing that's ready to run.

  return [self getSwarmActivity];
}

@end

